@@ -21,6 +21,7 @@ import math
 import hashlib
 import time
 import re
+import collections
 
 # ---------------------
 # Third party libraries
@@ -131,13 +132,11 @@ def hash(filepath):
 	'''Compute a hash from the image'''
 	BLOCK_SIZE = 1048576 # 1MByte, the size of each read from the file
 	
-	# Using the maximun digest size of 64 bytes takes 4 times
-	# So we take a compromise of reducing to 16 bytes, 
-	# with an increasing risk of hash collisions 1/2^128 * 100%
 	# md5() was the fastest algorithm I've tried
+	# but I detected a collision, so I now use blake2b with twice the digest size
 	
-	#file_hash = hashlib.blake2b(digest_size=16)
-	file_hash = hashlib.md5()
+	file_hash = hashlib.blake2b(digest_size=32)
+	#file_hash = hashlib.md5()
 	with open(filepath, 'rb') as f:
 		block = f.read(BLOCK_SIZE) 
 		while len(block) > 0:
@@ -170,14 +169,27 @@ def image_session_state_reset(connection, session):
 	connection.commit()
 
 
+
+def detect_dupl_hashes(names_hashes_list):
+	hashes_list = [ item['hash'] for item in names_hashes_list ]
+	c = collections.Counter(hashes_list)
+	for hsh, count in c.most_common():
+		if count > 1:
+			for item in names_hashes_list:
+				if item['hash'] == hsh:
+					name = item['name']
+					log.error("Image {0} has a clashing hash {1} ".format(name, hsh))
+
+
 def work_dir_to_session(connection, work_dir, filt):
 	file_list  = glob.glob(os.path.join(work_dir, filt))
 	log.info("Found {0} candidates matching filter {1}.".format(len(file_list), filt))
 	log.info("Computing hashes. This may take a while")
-	names_list = [ {'name': os.path.basename(p), 'hash': hash(p)} for p in file_list ]
+	names_hashes_list = [ {'name': os.path.basename(p), 'hash': hash(p)} for p in file_list ]
+	detect_dupl_hashes(names_hashes_list)
 	cursor = connection.cursor()
 	cursor.execute("CREATE TEMP TABLE candidate_t (name TEXT, hash BLOB, PRIMARY KEY(hash))")
-	cursor.executemany("INSERT INTO candidate_t (name,hash) VALUES (:name,:hash)", names_list)
+	cursor.executemany("INSERT INTO candidate_t (name,hash) VALUES (:name,:hash)", names_hashes_list)
 	connection.commit()
 	# Common images to database and work-dir
 	cursor.execute(
@@ -482,6 +494,7 @@ def stats_update_db(connection, rows):
 			model               = :model,        -- EXIF
 			iso                 = :iso,          -- EXIF
 			tstamp              = :tstamp,       -- EXIF
+			night               = :night,        -- computed from EXIF
 			exptime             = :exptime,      -- EXIF
 			focal_length        = :focal_length, -- EXIF
 			f_number            = :f_number,     -- EXIF
@@ -532,6 +545,7 @@ def do_stats(connection, session, work_dir, options):
 		row['model']        = metadata['model']
 		row['iso']          = metadata['iso']
 		row['tstamp']       = metadata['tstamp']
+		row['night']        = metadata['night']
 		row['exptime']      = metadata['exptime']
 		row['bias']         = metadata['bias']
 		row['focal_length'] = metadata['focal_length']
@@ -800,10 +814,23 @@ EXPORT_HEADERS = [
 			'bias'           ,
 		]
 
+
+def night_iterable(connection, session):
+	row = {'session': session}
+	cursor = connection.cursor()
+	cursor.execute(
+		'''
+		SELECT  DISTINCT night
+		FROM image_t
+		WHERE session == :session
+		''', row)
+	return cursor
+
+
 # we are not using the image_v VIEW for the time being
 # We display the RAW data without dark and bias substraction
-def export_session_iterable(connection, session):
-	row = {'session': session, 'state': STATS_COMPUTED, 'type': LIGHT_FRAME}
+def export_session_iterable(connection, session, night):
+	row = {'session': session, 'state': STATS_COMPUTED, 'type': LIGHT_FRAME, 'night': night}
 	cursor = connection.cursor()
 	cursor.execute(
 		'''
@@ -837,9 +864,10 @@ def export_session_iterable(connection, session):
 				vari_dark_B4,   -- Array index 27
 				bias
 		FROM image_t
-		WHERE state >= :state
-		AND   type = :type
+		WHERE state  >= :state
+		AND   type    = :type
 		AND   session = :session
+		AND   night   = :night
 		ORDER BY tstamp ASC
 		''', row)
 	return cursor
@@ -901,15 +929,14 @@ def var2std(item):
 
 
 
-def get_file_path(connection, session, work_dir, options):
+def get_file_path(connection, night, work_dir, options):
 	# This is for automatic reductions mainly
 	key, ext  = os.path.splitext(options.config)
 	key       = os.path.basename(key)
-	parent    = os.path.dirname(work_dir)
-	wdtag     = os.path.basename(work_dir)
-	filename  = "-".join([key, wdtag + '.csv'])
+	#wdtag     = os.path.basename(work_dir)
+	filename  = "-".join([key, night + '.csv'])
 	if options.multiuser:
-		subdir = os.path.join(parent, options.csv_subdir)
+		subdir = os.path.join(options.csv_dir, key)
 		os.makedirs(subdir, exist_ok=True)
 		file_path = os.path.join(subdir, filename)
 	else:
@@ -921,18 +948,21 @@ def do_export_work_dir(connection, session, work_dir, options):
 	'''Export a working directory of image redictions to a single file'''
 	fieldnames = ["session","observer","organization","location","type"]
 	fieldnames.extend(EXPORT_HEADERS)
-	if session_processed(connection, session):
+	if not session_processed(connection, session):
+		log.info("No new CSV file generation")
+		return
+
+	for (night,) in night_iterable(connection, session):
 		# Write a session CSV file
-		session_csv_file = get_file_path(connection, session, work_dir, options)
+		session_csv_file = get_file_path(connection, night, work_dir, options)
 		with myopen(session_csv_file, 'w') as csvfile:
 			writer = csv.writer(csvfile, delimiter=';')
 			writer.writerow(fieldnames)
-			for row in export_session_iterable(connection, session):
+			for row in export_session_iterable(connection, session, night):
 				row = map(var2std, enumerate(row))
 				writer.writerow(row)
-		log.info("Saved data to session CSV file {0}".format(session_csv_file))
-	else:
-		log.info("No new CSV file generation")
+	log.info("Saved data to session CSV file {0}".format(session_csv_file))
+	
 	
 
 def do_export_all(connection,  options):
